--[[
	button.lua
		Scripts for Bongos Action Buttons, which add more features to blizzard action buttons
		
	Changes over the normal action buttons:
		Colored red when out of range
		Selfcast can be enabled
		Altcast can be enabled
		Hotkeys are shortened to N for NumPad, S for shift, A for Alt, and C for ctrl
		Added the abiliy to skip pages when paging.
			
	Saved Variables:
		Bongos.altCast - the key used for altcast
		Bongos.selfCast - the key used 
		
	TODO:
		Cleanup code
--]]

--local globals
bg_quickPaged = nil;

--local constants
MAX_BUTTONS = 120; --the current maximum amount of action buttons
local SHAPESHIFT_START = 5; --offset of shapeshift

--Create an Action Button with the given ID and parent
function BActionButton_Create(id, parent)
	local button = CreateFrame("CheckButton", "BActionButton" .. id, parent, "ActionButtonTemplate");
	button:SetID(id);
	
	getglobal("BActionButton" .. id .. "NormalTexture"):SetVertexColor(1.0, 1.0, 1.0, 0.5);
	
	if(Bongos.hideHotkeys) then
		getglobal("BActionButton" .. id .. "HotKey"):Hide();
	end
	if(Bongos.hideMacro) then
		getglobal("BActionButton" .. id .. "Name"):Hide();
	end
	
	BActionButton_SetScripts(button);
	
	return button;
end

--Pretty much acts as a loading function, sets all scripts, and event listeners.
function BActionButton_SetScripts(button)
	button:RegisterForDrag("LeftButton", "RightButton");
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	
	button.flashing = 0;
	button.flashtime = 0;
	
	button:SetScript("OnUpdate", BActionButton_OnUpdate);
	button:SetScript("OnClick", BActionButton_OnClick);
	button:SetScript("OnDragStart", BActionButton_OnDragStart);
	button:SetScript("OnReceiveDrag", BActionButton_OnReceiveDrag);
	button:SetScript("OnEnter", BActionButton_OnEnter);
	button:SetScript("OnLeave", BActionButton_OnLeave);
	
	BActionButton_UpdateAll(button);
	Bongos_UpdateHotkey(button);
end

--[[
	Button Actions
		These are called when pressing and releasing a button, as well as using a hotkey
--]]

local function ShouldSelfCast()
	return (Bongos.selfCast and not( UnitExists("target") and SpellCanTargetUnit("target") ) );
end

--Uses a button's action
--There's three modes: selfcast - no target 
function BActionButton_UseAction(id, onSelf)
	if( Bongos_IsSelfCastHotkeyDown() ) then
		UseAction(id, 0, 1);
		if( SpellIsTargeting() ) then
			SpellTargetUnit("player");
		end
	else
		UseAction(id, 0, onSelf);
		if( SpellIsTargeting() and ShouldSelfCast() ) then
			SpellTargetUnit("player");
		end
	end
end

--[[ These two functions override ActionButtonUp and ActionButtonDown ]]--
--Release button (uses an ability)
function BActionButtonUp(id, onSelf)
	local barNum = math.ceil(id / 12);
	local button = getglobal("BActionButton".. id);
	
	-- Used to save a macro
	if ( MacroFrame_SaveMacro ) then
		MacroFrame_SaveMacro();
	end
		
	local pagedID = BActionButton_GetPagedID(id);
	
	if(GetLocale() == "zhCN") then
		if ( button and button:GetButtonState() == "PUSHED") then
			button:SetButtonState("NORMAL");
			button:SetChecked( IsCurrentAction( pagedID ) );
			BActionButton_UseAction(pagedID, onSelf);
		end
	else
		if ( button and button:GetButtonState() == "PUSHED") then
			button:SetButtonState("NORMAL");
			button:SetChecked( IsCurrentAction( pagedID ) );
		end
		BActionButton_UseAction(pagedID, onSelf);
	end
end

--Press button
function BActionButtonDown(id)
	local button = getglobal("BActionButton".. id);
	
	if ( button and button:GetButtonState() == "NORMAL" ) then
		if(not (Bongos.preventMash and bg_timeSinceLastHit and bg_timeSinceLastHit[id]) ) then
			button:SetButtonState("PUSHED");
		end
	end
end

--[[
	OnX functions
--]]

function BActionButton_OnClick()
	if ( Bongos_IsQuickMoveKeyDown() or bg_showGrid ) then
		local pagedID = BActionButton_GetPagedID( this:GetID() );
		PickupAction( pagedID );
	else
		if ( MacroFrame_SaveMacro ) then
			MacroFrame_SaveMacro();
		end
		BActionButton_UseAction( BActionButton_GetPagedID( this:GetID() ) );
	end
	BActionButton_UpdateState(this);
end

function BActionButton_OnDragStart()
	if ( not Bongos.buttonLocked or Bongos_IsQuickMoveKeyDown() or bg_showGrid ) then
		local pagedID = BActionButton_GetPagedID( this:GetID() );
		PickupAction( pagedID );
		BActionButton_UpdateState(this);
	end
end

function BActionButton_OnReceiveDrag()
	local pagedID = BActionButton_GetPagedID( this:GetID() );
	PlaceAction( pagedID );
	BActionButton_UpdateState(this);
end

function BActionButton_OnEnter()
	BActionButton_SetTooltip(this);
end

function BActionButton_OnLeave()
	this.updateTooltip = nil;
	GameTooltip:Hide();
end

--OnUpdate
function BActionButton_OnUpdate()	
	--update flashing
	if ( this.flashing == 1 ) then
		this.flashtime = this.flashtime - arg1;
		if ( this.flashtime <= 0 ) then
			local overtime = -this.flashtime;
			if ( overtime >= ATTACK_BUTTON_FLASH_TIME ) then
				overtime = 0;
			end
			this.flashtime = ATTACK_BUTTON_FLASH_TIME - overtime;

			local flashTexture = getglobal(this:GetName().."Flash");
			if ( flashTexture:IsVisible() ) then
				flashTexture:Hide();
			else
				flashTexture:Show();
			end
		end
	end
	
	-- Handle range indicator
	if ( this.rangeTimer ) then
		if ( this.rangeTimer < 0 ) then
		
			local hotkey = getglobal(this:GetName().."HotKey");
			local id = BActionButton_GetPagedID( this:GetID() );
			
			if ( IsActionInRange( id ) == 0  ) then
				hotkey:SetVertexColor(1.0, 0.1, 0.1);
				if ( Bongos.color and IsUsableAction( id ) ) then
					getglobal(this:GetName() .. "Icon"):SetVertexColor(Bongos.rangeColor.r, Bongos.rangeColor.g, Bongos.rangeColor.b);
				end
			else
				hotkey:SetVertexColor(0.6, 0.6, 0.6);
				if ( IsUsableAction(id) ) then
					getglobal(this:GetName() .. "Icon"):SetVertexColor(1, 1, 1);
				end
			end
			this.rangeTimer = TOOLTIP_UPDATE_TIME;
		else
			this.rangeTimer = this.rangeTimer - arg1;
		end
	end

	--Tooltip stuff, probably for the cooldown timer
	if ( this.updateTooltip ) then
		this.updateTooltip = this.updateTooltip - arg1;
		if ( this.updateTooltip <= 0 ) then
			if ( GameTooltip:IsOwned(this) ) then
				BActionButton_SetTooltip(this);
			else
				this.updateTooltip = nil;
			end
		end
	end
end

--[[ 
	Update Functions 
		These functions update certain information about a button
		
	All of these functions make the assumption that the button exists.
--]]

--Updates the icon, count, cooldown, usability color, if the button is flashing, if the button is equipped,  and macro text.
function BActionButton_Update(button)
	local pagedID = BActionButton_GetPagedID( button:GetID() );
	
	--[[
	-- Determine whether or not the button should be flashing or not since the button may have missed the enter combat event
	IN_ATTACK_MODE = ( IsAttackAction(pagedID) and IsCurrentAction(pagedID) );
	IN_AUTOREPEAT_MODE = IsAutoRepeatAction(pagedID); --]]
	
	local icon = getglobal(button:GetName().."Icon");
	local buttonCooldown = getglobal(button:GetName().."Cooldown");
	local texture = GetActionTexture(pagedID);
	
	--if the button's not empty, then show the icon, else show an empty button
	if ( texture ) then
		icon:SetTexture(texture);
		icon:Show();
		button.rangeTimer = TOOLTIP_UPDATE_TIME;
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
	else
		icon:Hide();
		buttonCooldown:Hide();
		button.rangeTimer = nil;
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot");
		getglobal(button:GetName().."HotKey"):SetVertexColor(0.6, 0.6, 0.6);
	end
	
	--update count
	if ( IsConsumableAction( pagedID ) ) then
		getglobal(button:GetName().."Count"):SetText( GetActionCount(pagedID) );
	else
		getglobal(button:GetName().."Count"):SetText("");
	end
	
	--update cooldown/usability if there's an action
	if ( HasAction(pagedID) ) then
		button:Show();
		BActionButton_UpdateUsable(button);
		BActionButton_UpdateCooldown(button);
	elseif ( not (bg_showGrid or Bongos.showGrid) ) then
		button:Hide();
	else
		button:Show();
		getglobal(button:GetName().."NormalTexture"):SetVertexColor(1.0, 1.0, 1.0, 0.5);
		buttonCooldown:Hide();
	end
	
	BActionButton_UpdateFlash(button);
	
	-- Add a green border if button is an equipped item
	local border = getglobal(button:GetName().."Border");
	if ( IsEquippedAction(pagedID) ) then
		border:SetVertexColor(0, 1.0, 0, 0.7);
		border:Show();
	else
		border:Hide();
	end
	
	if ( GameTooltip:IsOwned(button) ) then
		BActionButton_SetTooltip(button);
	else
		button.updateTooltip = nil;
	end

	-- Update Macro Text
	local macroName = getglobal(button:GetName().."Name");
	macroName:SetText(GetActionText(pagedID));
end

--colors the action button if out of mana, out of range, etc
function BActionButton_UpdateUsable(button)
	local icon = getglobal(button:GetName().."Icon");
	local isUsable, notEnoughMana = IsUsableAction( BActionButton_GetPagedID( button:GetID() ) );
	
	if ( isUsable ) then
		if ( Bongos.color and IsActionInRange( BActionButton_GetPagedID(button:GetID()) ) == 0) then
			icon:SetVertexColor(Bongos.rangeColor.r, Bongos.rangeColor.g, Bongos.rangeColor.b);
		else
			icon:SetVertexColor(1,1,1);
		end
	elseif ( notEnoughMana ) then
		--Make the icon blue if out of mana
		icon:SetVertexColor(0.5, 0.5, 1.0);
	else
		--Skill unusable
		icon:SetVertexColor(0.3, 0.3, 0.3);
	end
end

--Update the cooldown timer
function BActionButton_UpdateCooldown(button)
	local start, duration, enable = GetActionCooldown( BActionButton_GetPagedID( button:GetID() ) );
	CooldownFrame_SetTimer(getglobal(button:GetName().."Cooldown"), start, duration, enable);
end

--Update if a button is checked or not
function BActionButton_UpdateState(button)
	local id = BActionButton_GetPagedID( button:GetID() );
	button:SetChecked( IsCurrentAction(id) or IsAutoRepeatAction(id) );
end

--Update a a flashing button
function BActionButton_UpdateFlash(button)
	local pagedID = BActionButton_GetPagedID( button:GetID() );
	if ( (IsCurrentAction(pagedID) and IsAttackAction(pagedID)) or IsAutoRepeatAction(pagedID) ) then
		button.flashing = 1;
		button.flashtime = 0;
	else
		button.flashing = nil;
		getglobal(button:GetName().."Flash"):Hide();
	end
	BActionButton_UpdateState(button);
end

--next two are basically wrapper functions for use with ForAllButtons
function BActionButton_UpdateUsableAndCooldown(button)
	BActionButton_UpdateUsable(button);
	BActionButton_UpdateCooldown(button);
end

function BActionButton_UpdateAll(button)
	BActionButton_Update(button);
	BActionButton_UpdateState(button);
end

--hides a button, if its currently not attached to any actionbar
function BActionButton_ShowIfUsed(button)
	if( button:GetParent() ) then
		BActionButton_UpdateAll(button);
		if(not button:GetParent():IsShown() ) then
			button:Hide();
		end
	else
		button:Hide();
	end
end

--[[ 
	Tooltip Functions
--]]

function BActionButton_SetTooltip(button)
	if ( GetCVar("UberTooltips") == "1" ) then
		GameTooltip_SetDefaultAnchor(GameTooltip, button);
	else
		GameTooltip:SetOwner(button, "ANCHOR_RIGHT");
	end
	
	if ( GameTooltip:SetAction( BActionButton_GetPagedID( button:GetID() ) ) ) then
		button.updateTooltip = TOOLTIP_UPDATE_TIME;
	else
		button.updateTooltip = nil;
	end
end

--[[
	Button Toggles
		Again, these assume the button exists
--]]

--Show Empty Buttons
function BActionButton_ShowGrid(button)
	getglobal(button:GetName() .."NormalTexture"):SetVertexColor(1.0, 1.0, 1.0, 0.5);
	button:Show();
end

--Hide Empty Buttons
function BActionButton_HideGrid(button)
	if ( not HasAction( BActionButton_GetPagedID( button:GetID() ) ) ) then
		button:Hide();
	end
end

function BActionButton_HideMacroText(button)
	getglobal(button:GetName().."Name"):Hide();
end

function BActionButton_ShowMacroText(button)
	getglobal(button:GetName().."Name"):Show();
end

function BActionButton_HideHotkeys(button)
	getglobal(button:GetName().."HotKey"):Hide();
end

function BActionButton_ShowHotkeys(button)
	getglobal(button:GetName().."HotKey"):Show();
end


--[[
	Action ID Functions
--]]

--returns a number depending on what you're targeting
--enemy target - 0, no target/friendly target - 1
local function GetTargetOffset()
	if( not bg_quickPaged ) then
		if( UnitExists("target") and UnitCanAttack("player", "target") ) then
			return 0;
		end
	end
	return 1;
end

function BActionButton_GetParent(id)
	local button = getglobal("BActionButton" .. id);
	if(button and button:GetParent() ) then
		return button:GetParent().id;
	end
	
	local maxButtonsPerBar = math.floor(MAX_BUTTONS / Bongos.numActionBars);
	for i = 1, Bongos.numActionBars do
		if( id <= maxButtonsPerBar * i ) then
			return i;
		end
	end
	
	return nil;
end

--Paging, this extends it to page 10 and allows page skipping
function BActionButton_GetPagedID(buttonID, barID)
	if(not barID) then
		barID = BActionButton_GetParent(buttonID);
	end
	
	if(not barID) then
		return buttonID;
	end
	
	--If paged, then return the paged ID,
	if(Bongos[barID].paging and CURRENT_ACTIONBAR_PAGE ~= 1) then
		local pagedID = math.mod( buttonID + ( (CURRENT_ACTIONBAR_PAGE - 1) * Bongos.skip + (CURRENT_ACTIONBAR_PAGE - 1) )  * NUM_ACTIONBAR_BUTTONS  ,  MAX_BUTTONS )
		
		if(pagedID == 0) then
			return MAX_BUTTONS;
		end
		
		return pagedID;
	--If not paged, then return either the bonus, contextual, or normal button ID
	else
		--if shapeshifted, in a different stance, and going back to the first page, return the 
		if ( barID == 1 and GetBonusBarOffset() ~= 0 ) then
			return (buttonID + (SHAPESHIFT_START + GetBonusBarOffset()) * Bongos[barID].size);
		end
		
		--do contextual paging on a bar, but only if you aren't paged
		--contextual paging uses the main offset as enemy, and +1 for friendly/self/notarget
		if( Bongos[barID].inContext ) then
			local pagedID = math.mod( buttonID + GetTargetOffset() * Bongos[barID].size, MAX_BUTTONS );
			
			if(pagedID == 0) then
				return MAX_BUTTONS;
			end
			
			return pagedID;
		end
	end
	
	return buttonID;
end
