--[[
	bar.lua
		Scripts used by the Bongos Class Bar replacement
		
	Saved Variables:
		Bongos.class = {
			<All variables from BBar>
			space
				The spacing between action buttons, in pixels.  A nil value means that the bar is using default spacing
			rows
				How many rows the bar is organized into.
		}
	
	TODO:
	
--]]

--constants
local DEFAULT_SPACING = 2;
local DEFAULT_ROWS = 1;

local SetScripts, CreateClassButton;

--[[ 
	Class Button Functions
--]]

--constructor
function CreateClassButton(id, parent)
	--create the button
	local name = parent:GetName() .. "Button" .. id;
	local button = CreateFrame("CheckButton", name, parent);
	button:SetHeight(36);
	button:SetWidth(36);
	
	--[[ Textures ]]--
	
	--icon
	local iconTexture = button:CreateTexture(name .. "Icon", "BACKGROUND");
	iconTexture:SetAllPoints(button);
	
	--normal, pushed, highlight, checked
	local normalTexture = button:CreateTexture(name .. "NormalTexture");
	normalTexture:SetTexture("Interface\\Buttons\\UI-Quickslot2");
	normalTexture:SetHeight(52);
	normalTexture:SetWidth(52);
	normalTexture:SetPoint("CENTER", button, "CENTER", 0, -1);
	button:SetNormalTexture(normalTexture);
	
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress");
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
	button:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight");
	
	--[[ Hotkeys ]]--
	local hotkey = button:CreateFontString(name .. "HotKey", "ARTWORK");
	hotkey:SetFontObject(NumberFontNormalSmallGray);
	hotkey:SetPoint("TOPRIGHT", button, "TOPRIGHT", 2, -2);
	hotkey:SetJustifyH("RIGHT");
	hotkey:SetWidth(32);
	hotkey:SetHeight(10);
	
	if(Bongos.hideHotkeys) then
		hotkey:Hide();
	end
	
	--[[ Frames ]]--
	
	--cooldown model
	local cooldown = CreateFrame("Model", name .. "Cooldown", button);
	cooldown:SetAllPoints(button);
	cooldown:SetModel("Interface\\Cooldown\\UI-Cooldown-Indicator.mdx");
	cooldown:Hide();
	
	cooldown:SetScript("OnUpdateModel", CooldownFrame_OnUpdateModel);
	cooldown:SetScript("OnAnimFinished", CooldownFrame_OnAnimFinished);	
	
	button:SetID(id);
	button:SetAlpha(button:GetParent():GetAlpha());
	
	SetScripts(button);
	
	return button;
end

--load scripts
function SetScripts(button)
	button:SetScript("OnClick", BClassButton_OnClick);
	button:SetScript("OnEnter", BClassButton_OnEnter);
	button:SetScript("OnLeave", BClassButton_OnLeave);
end

--[[ OnX Functions ]]--
function BClassButton_OnClick()
	this:SetChecked(not this:GetChecked() );
	CastShapeshiftForm(this:GetID());
end

function BClassButton_OnEnter()
	if ( GetCVar("UberTooltips") == "1" ) then
		GameTooltip_SetDefaultAnchor(GameTooltip, this);
	else
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	end
	GameTooltip:SetShapeshift(this:GetID());
end

function BClassButton_OnLeave()
	GameTooltip:Hide();
end

--[[ 
	Class Bar Functions
 --]]
 
function BClassBar_Create()
	local bar = BBar_Create("BClassBar", "class");
	
	BClassBarDragButton:SetScript("OnClick", function()
		if(arg1=="RightButton") then
			BClassBar_ShowMenu();
		end
	end);
	BClassBar:SetScript("OnEvent", BClassBar_Update);
	BClassBar:SetScript("OnShow", BClassBar_Update);
	
	BClassBar:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");
	BClassBar:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
	BClassBar:RegisterEvent("UPDATE_INVENTORY_ALERTS");
	BClassBar:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	BClassBar:RegisterEvent("SPELL_UPDATE_USABLE");
	BClassBar:RegisterEvent("PLAYER_AURAS_CHANGED");
	BClassBar:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
	BClassBar:RegisterEvent("UNIT_HEALTH");
	BClassBar:RegisterEvent("UNIT_HEALTH");
	BClassBar:RegisterEvent("UNIT_RAGE");
	BClassBar:RegisterEvent("UNIT_FOCUS");
	BClassBar:RegisterEvent("UNIT_ENERGY");
	BClassBar:RegisterEvent("UPDATE_BINDINGS");
	
	if( not BClassBar:IsUserPlaced() ) then
		BClassBar:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 480);
	end
	
	return bar;
end

function BClassBar_Load()
	BClassBar_Create();
end

--[[ OnX Functions ]]--

--Update Function
function BClassBar_Update(bar)
	if(event == "UPDATE_BINDINGS") then
		BClassBar_UpdateBindings();
		return;
	end
	
	if(not bar) then
		bar = this;
	end
	local numForms = GetNumShapeshiftForms();
	
	--hide the bar if there's no Class forms
	if ( numForms > 0 and Bongos.class.vis ) then
		bar:Show();
	else
		bar:Hide();
		return;
	end
	
	local fileName, name, isActive, isCastable;
	local button, icon, cooldown;
	local start, duration, enable;
	local layoutChanged;
	
	--Update buttons
	for i=1, numForms do
		button = getglobal(bar:GetName() .. "Button" .. i);
		if(not button) then
			button = CreateClassButton(i, bar);
			layoutChanged = 1;
		end
		icon = getglobal(button:GetName() .. "Icon");
		
		texture, name, isActive, isCastable = GetShapeshiftFormInfo(i);
		icon:SetTexture(texture);

		--Cooldown stuffs
		cooldown = getglobal(button:GetName() .. "Cooldown");
		if ( texture ) then
			cooldown:Show();
		else
			cooldown:Hide();
		end
		start, duration, enable = GetShapeshiftFormCooldown(i);
		CooldownFrame_SetTimer(cooldown, start, duration, enable);
		button:SetChecked(isActive );
			
		if ( isCastable ) then
			icon:SetVertexColor(1.0, 1.0, 1.0);
		else
			icon:SetVertexColor(0.4, 0.4, 0.4);
		end
		button:Show();
	end
	
	if(layoutChanged) then
		BClassBar_Build(Bongos.class.rows, Bongos.class.space);
		BClassBar_UpdateBindings();
	end
end

function BClassBar_UpdateBindings()
	for i = 1, GetNumShapeshiftForms() do
		local button = getglobal("BClassBarButton" .. i);
		if(button) then
			Bongos_UpdateHotkey(button, "SHAPESHIFTBUTTON");
		else
			return;
		end
	end
end

function BClassBar_ShowHotkeys()
	for i = 1, GetNumShapeshiftForms() do
		local key = getglobal("BClassBarButton" .. i .. "HotKey");
		if(key) then
			key:Show();
		else
			return;
		end
	end
end

function BClassBar_HideHotkeys()
	for i = 1, GetNumShapeshiftForms() do
		local key = getglobal("BClassBarButton" .. i .. "HotKey");
		if(key) then
			key:Hide();
		else
			return;
		end
	end
end

--[[ Bar Layout Functions ]]--
local function SaveSettings(rows, space)
	if(not space or space == DEFAULT_SPACING) then
		space = DEFAULT_SPACING;
		Bongos.class.space = nil;
	else
		Bongos.class.space = space;
	end
	
	if(rows and rows > GetNumShapeshiftForms()) then
		rows = GetNumShapeshiftForms();
	end
	if(not rows or rows == DEFAULT_ROWS) then
		rows = DEFAULT_ROWS;
		Bongos.class.rows = nil;
	else
		Bongos.class.rows = rows;
	end
	
	return rows, space;
end

function BClassBar_Build(rows, space)
	rows, space = SaveSettings(rows, space);
	
	local numForms = GetNumShapeshiftForms();
	local columns = math.ceil(numForms / rows);
	
	--resize the bar
	BClassBar:SetWidth(1 + (36 + space) * columns - space);
	BClassBar:SetHeight(2 + (36 + space) * math.ceil(numForms / columns) - space);
	
	local index = 1;
	--set the position of the first button of the bar
	button = getglobal("BClassBarButton" .. index);
	button:ClearAllPoints();
	button:SetPoint("TOPLEFT", BClassBar, "TOPLEFT", 0, -1);
	
	--set the positions of the remaining buttons
	local i, j;
	for i = 1, rows, 1 do 
		for j = 1, columns, 1 do
			index = index + 1;
			if(index > numForms) then 
				return; 
			end
			
			button = getglobal("BClassBarButton" .. index);
			button:ClearAllPoints();
			button:SetPoint("LEFT", "BClassBarButton" .. index - 1, "RIGHT", space, 0);
		end
		if(index > numForms) then 
			return; 
		end
		
		button = getglobal("BClassBarButton" .. index);
		button:ClearAllPoints();
		button:SetPoint("TOP", "BClassBarButton" .. index - columns, "BOTTOM", 0, -space);
	end
end

--[[  Rightclick Menu Functions  ]]--

local function CreateConfigMenu(name)
	local menu = CreateFrame("Button", name, UIParent, "BongosRightClickMenuHide");
	menu:SetText("Class Bar");
	menu:SetWidth(220);
	menu:SetHeight(236);
	
	local rowsSlider = CreateFrame("Slider", name .. "Rows", menu, "BongosSlider");
	rowsSlider:SetPoint("TOPLEFT", name .. "Hide", "BOTTOMLEFT", 2, -10);
	rowsSlider:SetScript("OnValueChanged", function()
		if(not this:GetParent().onShow) then
			BClassBar_Build( this:GetValue(), Bongos.class.space );
		end
		getglobal(this:GetName() .. "ValText"):SetText( this:GetValue() );
	end);
	rowsSlider:SetValueStep(1);
	rowsSlider:SetMinMaxValues( 1, GetNumShapeshiftForms() );
	getglobal(name .. "RowsText"):SetText("Rows");
	getglobal(name .. "RowsLow"):SetText(1);
	
	local spacingSlider = CreateFrame("Slider", name .. "Spacing", menu, "BongosSpaceSlider");
	spacingSlider:SetPoint("TOP", rowsSlider, "BOTTOM", 0, -24);
	spacingSlider:SetScript("OnValueChanged", function()
		if(not this:GetParent().onShow) then
			BClassBar_Build( Bongos.class.rows, this:GetValue() );
		end
		getglobal(this:GetName() .. "ValText"):SetText( this:GetValue() );
	end);
	
	local scaleSlider = CreateFrame("Slider", name .. "Scale", menu, "BongosScaleSlider");
	scaleSlider:SetPoint("TOP", spacingSlider, "BOTTOM", 0, -24);
	
	local opacitySlider = CreateFrame("Slider", name .. "Opacity", menu, "BongosOpacitySlider");
	opacitySlider:SetPoint("TOP", scaleSlider, "BOTTOM", 0, -24);
end

--Called when the right click menu is shown, loads the correct values to the checkbuttons/sliders/text
function BClassBar_ShowMenu()
	if(not BongosClassBarMenu) then
		CreateConfigMenu("BongosClassBarMenu");
	end
	
	BongosClassBarMenu.onShow = 1;
	BongosClassBarMenu.frame = this:GetParent();
	
	BongosClassBarMenuHide:SetChecked(not Bongos.class.vis);

	if(Bongos.class.space) then
		BongosClassBarMenuSpacing:SetValue(Bongos.class.space);
	else
		BongosClassBarMenuSpacing:SetValue(DEFAULT_SPACING);
	end
	
	if(Bongos.class.rows) then
		BongosClassBarMenuRows:SetValue(Bongos.class.rows);
	else
		BongosClassBarMenuRows:SetValue(DEFAULT_ROWS);
	end
	BongosClassBarMenuRowsHigh:SetText( GetNumShapeshiftForms() );
	BongosClassBarMenuRows:SetMinMaxValues( 1, GetNumShapeshiftForms() );
	
	BongosClassBarMenuScale:SetValue( BClassBar:GetScale() * 100 );
	BongosClassBarMenuOpacity:SetValue( BClassBar:GetAlpha() * 100 );
	
	--Position menu then show it
	BMenu_ShowMenuForBar(BongosClassBarMenu, BClassBar);
	BongosClassBarMenu.onShow = nil;
end